@@ -1,7 +1,7 @@
 /*
  * This file is part of the libCEC(R) library.
  *
- * libCEC(R) is Copyright (C) 2011 Pulse-Eight Limited.  All rights reserved.
+ * libCEC(R) is Copyright (C) 2011-2012 Pulse-Eight Limited.  All rights reserved.
  * libCEC(R) is an original work, containing original code.
  *
  * libCEC(R) is a trademark of Pulse-Eight Limited.
@@ -30,7 +30,7 @@
  *     http://www.pulse-eight.net/
  */
 
-#include <cec.h>
+#include "../../include/cec.h"
 
 #include <cstdio>
 #include <fcntl.h>
@@ -45,32 +45,30 @@ using namespace CEC;
 using namespace std;
 using namespace PLATFORM;
 
-#define CEC_TEST_CLIENT_VERSION 1
-
 #include <cecloader.h>
 
+ICECCallbacks        g_callbacks;
+libcec_configuration g_config;
 int                  g_cecLogLevel(CEC_LOG_ALL);
 ofstream             g_logOutput;
 bool                 g_bShortLog(false);
 CStdString           g_strPort;
-uint8_t              g_iHDMIPort(CEC_DEFAULT_HDMI_PORT);
-cec_logical_address  g_iBaseDevice((cec_logical_address)CEC_DEFAULT_BASE_DEVICE);
-cec_device_type_list g_typeList;
 bool                 g_bSingleCommand(false);
 bool                 g_bExit(false);
 bool                 g_bHardExit(false);
 CMutex               g_outputMutex;
-ICECCallbacks        g_callbacks;
 
-inline void PrintToStdOut(const char *strOut)
+inline void PrintToStdOut(const char *strFormat, ...)
 {
+  CStdString strLog;
+
+  va_list argList;
+  va_start(argList, strFormat);
+  strLog.FormatV(strFormat, argList);
+  va_end(argList);
+
   CLockObject lock(g_outputMutex);
-  cout << strOut << endl;
-}
-
-inline void PrintToStdOut(const CStdString &strOut)
-{
-  PrintToStdOut(strOut.c_str());
+  cout << strLog << endl;
 }
 
 inline bool HexStrToInt(const std::string& data, uint8_t& value)
@@ -153,7 +151,7 @@ int CecLogMessage(void *UNUSED(cbParam), const cec_log_message &message)
 
     CStdString strFullLog;
     strFullLog.Format("%s[%16lld]\t%s", strLevel.c_str(), message.time, message.message);
-    PrintToStdOut(strFullLog);
+    PrintToStdOut(strFullLog.c_str());
 
     if (g_logOutput.is_open())
     {
@@ -182,28 +180,23 @@ void EnableCallbacks(ICECAdapter *adapter)
   g_callbacks.CBCecLogMessage = &CecLogMessage;
   g_callbacks.CBCecKeyPress   = &CecKeyPress;
   g_callbacks.CBCecCommand    = &CecCommand;
+  g_callbacks.CBCecConfigurationChanged = NULL;
   adapter->EnableCallbacks(NULL, &g_callbacks);
 }
 
 void ListDevices(ICECAdapter *parser)
 {
   cec_adapter *devices = new cec_adapter[10];
-  uint8_t iDevicesFound = parser->FindAdapters(devices, 10, NULL);
+  int8_t iDevicesFound = parser->FindAdapters(devices, 10, NULL);
   if (iDevicesFound <= 0)
   {
     PrintToStdOut("Found devices: NONE");
   }
   else
   {
-    CStdString strLog;
-    strLog.Format("Found devices: %d", iDevicesFound);
-    PrintToStdOut(strLog);
-    for (unsigned int iDevicePtr = 0; iDevicePtr < iDevicesFound; iDevicePtr++)
-    {
-      CStdString strDevice;
-      strDevice.Format("device:        %d\npath:          %s\ncom port:      %s", iDevicePtr + 1, devices[iDevicePtr].path, devices[iDevicePtr].comm);
-      PrintToStdOut(strDevice);
-    }
+    PrintToStdOut("Found devices: %d\n", iDevicesFound);
+    for (int8_t iDevicePtr = 0; iDevicePtr < iDevicesFound; iDevicePtr++)
+      PrintToStdOut("device:        %d\npath:          %s\ncom port:      %s\n", iDevicePtr + 1, devices[iDevicePtr].path, devices[iDevicePtr].comm);
   }
 }
 
@@ -234,26 +227,6 @@ void ShowHelpCommandLine(const char* strExec)
       "available commands" << endl;
 }
 
-ICECAdapter *CreateParser(cec_device_type_list typeList)
-{
-  ICECAdapter *parser = LibCecInit("CECTester", typeList);
-  if (!parser || parser->GetMinLibVersion() > CEC_TEST_CLIENT_VERSION)
-  {
-  #ifdef __WINDOWS__
-    PrintToStdOut("Cannot load libcec.dll");
-  #else
-    PrintToStdOut("Cannot load libcec.so");
-  #endif
-    return NULL;
-  }
-
-  CStdString strLog;
-  strLog.Format("CEC Parser created - libcec version %d.%d", parser->GetLibVersionMajor(), parser->GetLibVersionMinor());
-  PrintToStdOut(strLog.c_str());
-
-  return parser;
-}
-
 void ShowHelpConsole(void)
 {
   CLockObject lock(g_outputMutex);
@@ -279,9 +252,12 @@ void ShowHelpConsole(void)
   "[lad]                     lists active devices on the bus" << endl <<
   "[ad] {addr}               checks whether the specified device is active." << endl <<
   "[at] {type}               checks whether the specified device type is active." << endl <<
+  "[sp] {addr}               makes the specified physical address active." << endl <<
+  "[spl] {addr}              makes the specified logical address active." << endl <<
   "[volup]                   send a volume up command to the amp if present" << endl <<
   "[voldown]                 send a volume down command to the amp if present" << endl <<
   "[mute]                    send a mute/unmute command to the amp if present" << endl <<
+  "[self]                    show the list of addresses controlled by libCEC" << endl <<
   "[scan]                    scan the CEC bus and display device info" << endl <<
   "[mon] {1|0}               enable or disable CEC bus monitoring." << endl <<
   "[log] {1 - 31}            change the log level. see cectypes.h for values." << endl <<
@@ -295,6 +271,64 @@ void ShowHelpConsole(void)
   "================================================================================" << endl;
 }
 
+bool ProcessCommandSELF(ICECAdapter *parser, const string &command, string & UNUSED(arguments))
+{
+  if (command == "self")
+  {
+    cec_logical_addresses addr = parser->GetLogicalAddresses();
+    CStdString strOut = "Addresses controlled by libCEC: ";
+    bool bFirst(true);
+    for (uint8_t iPtr = 0; iPtr <= 15; iPtr++)
+    {
+      if (addr[iPtr])
+      {
+        strOut.AppendFormat((bFirst ? "%d%s" : ", %d%s"), iPtr, parser->IsActiveSource((cec_logical_address)iPtr) ? "*" : "");
+        bFirst = false;
+      }
+    }
+    PrintToStdOut(strOut.c_str());
+    return true;
+  }
+
+  return false;
+}
+
+bool ProcessCommandSP(ICECAdapter *parser, const string &command, string &arguments)
+{
+  if (command == "sp")
+  {
+    string strAddress;
+    int iAddress;
+    if (GetWord(arguments, strAddress))
+    {
+      sscanf(strAddress.c_str(), "%x", &iAddress);
+      if (iAddress >= 0 && iAddress <= 0xFFFF)
+        parser->SetStreamPath((uint16_t)iAddress);
+      return true;
+    }
+  }
+
+  return false;
+}
+
+bool ProcessCommandSPL(ICECAdapter *parser, const string &command, string &arguments)
+{
+  if (command == "spl")
+  {
+    string strAddress;
+    cec_logical_address iAddress;
+    if (GetWord(arguments, strAddress))
+    {
+      iAddress = (cec_logical_address)atoi(strAddress.c_str());
+      if (iAddress >= CECDEVICE_TV && iAddress < CECDEVICE_BROADCAST)
+        parser->SetStreamPath(iAddress);
+      return true;
+    }
+  }
+
+  return false;
+}
+
 bool ProcessCommandTX(ICECAdapter *parser, const string &command, string &arguments)
 {
   if (command == "tx" || command == "txn")
@@ -482,9 +516,7 @@ bool ProcessCommandVOLUP(ICECAdapter *parser, const string &command, string & UN
 {
   if (command == "volup")
   {
-    CStdString strLog;
-    strLog.Format("volume up: %2X", parser->VolumeUp());
-    PrintToStdOut(strLog);
+    PrintToStdOut("volume up: %2X", parser->VolumeUp());
     return true;
   }
 
@@ -495,9 +527,7 @@ bool ProcessCommandVOLDOWN(ICECAdapter *parser, const string &command, string &
 {
   if (command == "voldown")
   {
-    CStdString strLog;
-    strLog.Format("volume down: %2X", parser->VolumeDown());
-    PrintToStdOut(strLog);
+    PrintToStdOut("volume down: %2X", parser->VolumeDown());
     return true;
   }
 
@@ -508,9 +538,7 @@ bool ProcessCommandMUTE(ICECAdapter *parser, const string &command, string & UNU
 {
   if (command == "mute")
   {
-    CStdString strLog;
-    strLog.Format("mute: %2X", parser->MuteAudio());
-    PrintToStdOut(strLog);
+    PrintToStdOut("mute: %2X", parser->MuteAudio());
     return true;
   }
 
@@ -584,9 +612,7 @@ bool ProcessCommandVEN(ICECAdapter *parser, const string &command, string &argum
       if (iDev >= 0 && iDev < 15)
       {
         uint64_t iVendor = parser->GetDeviceVendorId((cec_logical_address) iDev);
-        CStdString strLog;
-        strLog.Format("vendor id: %06x", iVendor);
-        PrintToStdOut(strLog);
+        PrintToStdOut("vendor id: %06x", iVendor);
         return true;
       }
     }
@@ -606,9 +632,7 @@ bool ProcessCommandVER(ICECAdapter *parser, const string &command, string &argum
       if (iDev >= 0 && iDev < 15)
       {
         cec_version iVersion = parser->GetDeviceCecVersion((cec_logical_address) iDev);
-        CStdString strLog;
-        strLog.Format("CEC version %s", parser->ToString(iVersion));
-        PrintToStdOut(strLog);
+        PrintToStdOut("CEC version %s", parser->ToString(iVersion));
         return true;
       }
     }
@@ -628,9 +652,7 @@ bool ProcessCommandPOW(ICECAdapter *parser, const string &command, string &argum
       if (iDev >= 0 && iDev < 15)
       {
         cec_power_status iPower = parser->GetDevicePowerStatus((cec_logical_address) iDev);
-        CStdString strLog;
-        strLog.Format("power status: %s", parser->ToString(iPower));
-        PrintToStdOut(strLog);
+        PrintToStdOut("power status: %s", parser->ToString(iPower));
         return true;
       }
     }
@@ -650,9 +672,7 @@ bool ProcessCommandNAME(ICECAdapter *parser, const string &command, string &argu
       if (iDev >= 0 && iDev < 15)
       {
         cec_osd_name name = parser->GetDeviceOSDName((cec_logical_address)iDev);
-        CStdString strLog;
-        strLog.Format("OSD name of device %d is '%s'", iDev, name.name);
-        PrintToStdOut(strLog);
+        PrintToStdOut("OSD name of device %d is '%s'", iDev, name.name);
       }
       return true;
     }
@@ -665,14 +685,12 @@ bool ProcessCommandLAD(ICECAdapter *parser, const string &command, string & UNUS
 {
   if (command == "lad")
   {
-    CStdString strLog;
     PrintToStdOut("listing active devices:");
     cec_logical_addresses addresses = parser->GetActiveDevices();
     for (uint8_t iPtr = 0; iPtr <= 11; iPtr++)
       if (addresses[iPtr])
       {
-        strLog.Format("logical address %X", (int)iPtr);
-        PrintToStdOut(strLog);
+        PrintToStdOut("logical address %X", (int)iPtr);
       }
     return true;
   }
@@ -689,11 +707,7 @@ bool ProcessCommandAD(ICECAdapter *parser, const string &command, string &argume
     {
       int iDev = atoi(strDev);
       if (iDev >= 0 && iDev < 15)
-      {
-        CStdString strLog;
-        strLog.Format("logical address %X is %s", iDev, (parser->IsActiveDevice((cec_logical_address)iDev) ? "active" : "not active"));
-        PrintToStdOut(strLog);
-      }
+        PrintToStdOut("logical address %X is %s", iDev, (parser->IsActiveDevice((cec_logical_address)iDev) ? "active" : "not active"));
     }
   }
 
@@ -716,9 +730,8 @@ bool ProcessCommandAT(ICECAdapter *parser, const string &command, string &argume
         type = CEC_DEVICE_TYPE_RECORDING_DEVICE;
       else if (strType.Equals("t"))
         type = CEC_DEVICE_TYPE_TUNER;
-      CStdString strLog;
-      strLog.Format("device %d is %s", type, (parser->IsActiveDeviceType(type) ? "active" : "not active"));
-      PrintToStdOut(strLog);
+
+      PrintToStdOut("device %d is %s", type, (parser->IsActiveDeviceType(type) ? "active" : "not active"));
       return true;
     }
   }
@@ -766,9 +779,8 @@ bool ProcessCommandLOG(ICECAdapter * UNUSED(parser), const string &command, stri
       if (iNewLevel >= CEC_LOG_ERROR && iNewLevel <= CEC_LOG_ALL)
       {
         g_cecLogLevel = iNewLevel;
-        CStdString strLog;
-        strLog.Format("log level changed to %s", strLevel.c_str());
-        PrintToStdOut(strLog);
+
+        PrintToStdOut("log level changed to %s", strLevel.c_str());
         return true;
       }
     }
@@ -856,7 +868,10 @@ bool ProcessConsoleCommand(ICECAdapter *parser, string &input)
       ProcessCommandR(parser, command, input) ||
       ProcessCommandH(parser, command, input) ||
       ProcessCommandLOG(parser, command, input) ||
-      ProcessCommandSCAN(parser, command, input);
+      ProcessCommandSCAN(parser, command, input) ||
+      ProcessCommandSP(parser, command, input) ||
+      ProcessCommandSPL(parser, command, input) ||
+      ProcessCommandSELF(parser, command, input);
     }
   }
   return true;
@@ -920,25 +935,25 @@ bool ProcessCommandLineArguments(int argc, char *argv[])
           {
             if (!g_bSingleCommand)
               cout << "== using device type 'playback device'" << endl;
-            g_typeList.add(CEC_DEVICE_TYPE_PLAYBACK_DEVICE);
+            g_config.deviceTypes.add(CEC_DEVICE_TYPE_PLAYBACK_DEVICE);
           }
           else if (!strcmp(argv[iArgPtr + 1], "r"))
           {
             if (!g_bSingleCommand)
               cout << "== using device type 'recording device'" << endl;
-            g_typeList.add(CEC_DEVICE_TYPE_RECORDING_DEVICE);
+            g_config.deviceTypes.add(CEC_DEVICE_TYPE_RECORDING_DEVICE);
           }
           else if (!strcmp(argv[iArgPtr + 1], "t"))
           {
             if (!g_bSingleCommand)
               cout << "== using device type 'tuner'" << endl;
-            g_typeList.add(CEC_DEVICE_TYPE_TUNER);
+            g_config.deviceTypes.add(CEC_DEVICE_TYPE_TUNER);
           }
           else if (!strcmp(argv[iArgPtr + 1], "a"))
           {
             if (!g_bSingleCommand)
               cout << "== using device type 'audio system'" << endl;
-            g_typeList.add(CEC_DEVICE_TYPE_AUDIO_SYSTEM);
+            g_config.deviceTypes.add(CEC_DEVICE_TYPE_AUDIO_SYSTEM);
           }
           else
           {
@@ -951,7 +966,7 @@ bool ProcessCommandLineArguments(int argc, char *argv[])
       else if (!strcmp(argv[iArgPtr], "--list-devices") ||
                !strcmp(argv[iArgPtr], "-l"))
       {
-        ICECAdapter *parser = CreateParser(g_typeList);
+        ICECAdapter *parser = LibCecInitialise(&g_config);
         if (parser)
         {
           ListDevices(parser);
@@ -960,6 +975,11 @@ bool ProcessCommandLineArguments(int argc, char *argv[])
         }
         bReturn = false;
       }
+      else if (!strcmp(argv[iArgPtr], "--bootloader"))
+      {
+        LibCecBootloader();
+        bReturn = false;
+      }
       else if (!strcmp(argv[iArgPtr], "--single-command") ||
           !strcmp(argv[iArgPtr], "-s"))
       {
@@ -977,8 +997,8 @@ bool ProcessCommandLineArguments(int argc, char *argv[])
       {
         if (argc >= iArgPtr + 2)
         {
-          g_iBaseDevice = (cec_logical_address)atoi(argv[iArgPtr + 1]);
-          cout << "using base device '" << (int)g_iBaseDevice << "'" << endl;
+          g_config.baseDevice = (cec_logical_address)atoi(argv[iArgPtr + 1]);
+          cout << "using base device '" << (int)g_config.baseDevice << "'" << endl;
           ++iArgPtr;
         }
         ++iArgPtr;
@@ -988,8 +1008,8 @@ bool ProcessCommandLineArguments(int argc, char *argv[])
       {
         if (argc >= iArgPtr + 2)
         {
-          g_iHDMIPort = (int8_t)atoi(argv[iArgPtr + 1]);
-          cout << "using HDMI port '" << (int)g_iHDMIPort << "'" << endl;
+          g_config.iHDMIPort = (int8_t)atoi(argv[iArgPtr + 1]);
+          cout << "using HDMI port '" << (int)g_config.iHDMIPort << "'" << endl;
           ++iArgPtr;
         }
         ++iArgPtr;
@@ -1006,20 +1026,27 @@ bool ProcessCommandLineArguments(int argc, char *argv[])
 
 int main (int argc, char *argv[])
 {
-  g_typeList.clear();
+  g_config.Clear();
+  snprintf(g_config.strDeviceName, 13, "CECTester");
+  g_config.callbackParam      = NULL;
+  g_config.clientVersion      = CEC_CLIENT_VERSION_1_5_0;
+  g_callbacks.CBCecLogMessage = &CecLogMessage;
+  g_callbacks.CBCecKeyPress   = &CecKeyPress;
+  g_callbacks.CBCecCommand    = &CecCommand;
+  g_config.callbacks          = &g_callbacks;
 
   if (!ProcessCommandLineArguments(argc, argv))
     return 0;
 
-  if (g_typeList.IsEmpty())
+  if (g_config.deviceTypes.IsEmpty())
   {
     if (!g_bSingleCommand)
       cout << "No device type given. Using 'recording device'" << endl;
-    g_typeList.add(CEC_DEVICE_TYPE_RECORDING_DEVICE);
+    g_config.deviceTypes.add(CEC_DEVICE_TYPE_RECORDING_DEVICE);
   }
 
-  ICECAdapter *parser = LibCecInit("CECTester", g_typeList);
-  if (!parser || parser->GetMinLibVersion() > CEC_TEST_CLIENT_VERSION)
+  ICECAdapter *parser = LibCecInitialise(&g_config);
+  if (!parser)
   {
 #ifdef __WINDOWS__
     cout << "Cannot load libcec.dll" << endl;
@@ -1036,7 +1063,7 @@ int main (int argc, char *argv[])
   if (!g_bSingleCommand)
   {
     CStdString strLog;
-    strLog.Format("CEC Parser created - libcec version %d.%d", parser->GetLibVersionMajor(), parser->GetLibVersionMinor());
+    strLog.Format("CEC Parser created - libCEC version %s", parser->ToString((cec_server_version)g_config.serverVersion));
     cout << strLog.c_str() << endl;
 
     //make stdin non-blocking
@@ -1072,29 +1099,17 @@ int main (int argc, char *argv[])
     }
   }
 
-  EnableCallbacks(parser);
-
-  parser->SetHDMIPort(g_iBaseDevice, g_iHDMIPort);
   PrintToStdOut("opening a connection to the CEC adapter...");
 
   if (!parser->Open(g_strPort.c_str()))
   {
-    CStdString strLog;
-    strLog.Format("unable to open the device on port %s", g_strPort.c_str());
-    PrintToStdOut(strLog);
+    PrintToStdOut("unable to open the device on port %s", g_strPort.c_str());
     UnloadLibCec(parser);
     return 1;
   }
 
   if (!g_bSingleCommand)
-  {
-    PrintToStdOut("cec device opened");
-
-    parser->PowerOnDevices(CECDEVICE_TV);
-    parser->SetActiveSource();
-
     PrintToStdOut("waiting for input");
-  }
 
   while (!g_bExit && !g_bHardExit)
   {
@@ -1111,12 +1126,9 @@ int main (int argc, char *argv[])
       g_bExit = true;
 
     if (!g_bExit && !g_bHardExit)
-      CCondition::Sleep(50);
+      CEvent::Sleep(50);
   }
 
-  if (!g_bSingleCommand && !g_bHardExit)
-    parser->StandbyDevices(CECDEVICE_BROADCAST);
-
   parser->Close();
   UnloadLibCec(parser);
 
